﻿Public Class Form1

    Property radius As Integer = 100
    Property angle As Integer = 90

    Dim centre As New Point(376, 376)
    Dim dragHandleCentre As New Point(470, 370)

    Private WithEvents sizer As New DragHandle

    ''' <summary>
    ''' Sets up designer form for use
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Me.Controls.Add(sizer)
        sizer.Location = New Point(470, 370)
        Me.DoubleBuffered = True
        cboTextColor.DataSource = New ColorComboBoxItem() {New ColorComboBoxItem("Black", Color.Black),
                                                            New ColorComboBoxItem("White", Color.White)}
        cboBackColor.DataSource = New ColorComboBoxItem() {New ColorComboBoxItem("Red", Color.FromArgb(255, 140, 150)),
                                                            New ColorComboBoxItem("Green", Color.FromArgb(145, 255, 145)),
                                                            New ColorComboBoxItem("Blue", Color.FromArgb(130, 130, 255))}
    End Sub

    ''' <summary>
    ''' Handles sizer dragging
    ''' Resizes the arrow pointer and redirections the angle of the pointer
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub sizer_Move(sender As Object, e As EventArgs) Handles sizer.Move
        dragHandleCentre = New Point(sizer.Left + 6, sizer.Top + 6)
        Me.radius = Measurement.lineLength(centre, dragHandleCentre)
        Me.angle = Measurement.FindAngle(centre, dragHandleCentre)
        lblPointerAngle.Text = $"Pointer angle: {Me.pointerAngle}"
        Me.Invalidate()
    End Sub

    ''' <summary>
    ''' Form1_Paint
    ''' Redraws the designer form
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub Form1_Paint(sender As Object, e As PaintEventArgs) Handles Me.Paint
        e.Graphics.CompositingQuality = Drawing2D.CompositingQuality.HighQuality
        e.Graphics.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic
        e.Graphics.DrawEllipse(Pens.Black, New Rectangle(centre.X - Me.radius, centre.Y - Me.radius, Me.radius * 2, Me.radius * 2))
        e.Graphics.TranslateTransform(376, 376)
        e.Graphics.RotateTransform(Me.angle - 90)
        Dim img As Bitmap = arrowImage()
        e.Graphics.DrawImage(img, New Point(0, -(Math.Floor(img.Height / 2) + 1)))
        e.Graphics.ResetTransform()
    End Sub

    ''' <summary>
    ''' cboTextColor_SelectedIndexChanged
    ''' Changes txtCaption.ForeColor and invokes Form_Paint
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub cboTextColor_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cboTextColor.SelectedIndexChanged
        txtCaption.ForeColor = DirectCast(cboTextColor.SelectedItem, ColorComboBoxItem).color
        Me.Invalidate()
    End Sub

    ''' <summary>
    ''' cboBackColor_SelectedIndexChanged
    ''' Changes txtCaption.BackColor and invokes Form_Paint
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub cboBackColor_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cboBackColor.SelectedIndexChanged
        txtCaption.BackColor = DirectCast(cboBackColor.SelectedItem, ColorComboBoxItem).color
        Me.Invalidate()
    End Sub

    ''' <summary>
    ''' Resizes the thickness of the arrow pointer
    ''' and invokes Form_Paint
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub nudThickness_ValueChanged(sender As Object, e As EventArgs) Handles nudThickness.ValueChanged
        Me.Invalidate()
    End Sub

    ''' <summary>
    ''' Changes the arrow caption and invokes Form_Paint
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub txtCaption_TextChanged(sender As Object, e As EventArgs) Handles txtCaption.TextChanged
        Me.Invalidate()
    End Sub

    ''' <summary>
    ''' Saves the arrow pointer to a semi-transparent PNG file
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub btnSave_Click(sender As Object, e As EventArgs) Handles btnSave.Click
        'declare new SaveFileDialog + set it's initial properties
        Dim sfd As New SaveFileDialog With {
        .Title = "Choose file to save to",
        .Filter = "PNG (*.png)|*.png",
        .FilterIndex = 0,
        .InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop)}

        'show the dialog + display the results in a msgbox unless cancelled
        If sfd.ShowDialog = DialogResult.OK Then
            outputImage().Save(sfd.FileName, Drawing.Imaging.ImageFormat.Png)
        End If
    End Sub

    ''' <summary>
    ''' pointerAngle() Function
    ''' </summary>
    ''' <returns>The angle the arrow pointer is pointed towards</returns>
    Private Function pointerAngle() As Integer
        If angle < 180 Then
            Return angle + 180
        Else
            Return angle - 180
        End If
    End Function

    ''' <summary>
    ''' arrowImage() Function
    ''' </summary>
    ''' <returns>An unrotated arrow pointer bitmap, as designed</returns>
    Private Function arrowImage() As Bitmap
        Dim aw As Integer = Math.Floor(nudThickness.Value / 4)
        Dim ah As Integer = 50 + Math.Floor(nudThickness.Value / 5)
        Dim v As Integer = CInt(nudThickness.Value)

        Dim img As New Bitmap(Me.radius - 6, v + aw * 2)
        Dim gr As Graphics = Graphics.FromImage(img)
        gr.CompositingQuality = Drawing2D.CompositingQuality.HighQuality
        gr.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic

        Dim polygon() As Point = New Point() {New Point(0, (img.Height / 2) + 1),
                                                New Point(ah, 0),
                                                New Point(ah, aw),
                                                New Point(img.Width - 1, aw),
                                                New Point(img.Width - 1, img.Height - aw),
                                                New Point(ah, img.Height - aw),
                                                New Point(ah, img.Height),
                                                New Point(0, (img.Height / 2) + 1)}
        gr.FillPolygon(New SolidBrush(DirectCast(cboBackColor.SelectedItem, ColorComboBoxItem).color), polygon)
        gr.DrawPolygon(Pens.Black, polygon)

        Dim s() As String = txtCaption.Lines
        If s.Length > 0 Then
            Dim h As Integer = TextRenderer.MeasureText(s(0), Me.Font).Height
            Dim th As Integer = h * s.Length
            Dim tw As Integer = 0
            For Each line As String In s
                If TextRenderer.MeasureText(line, Me.Font).Width > tw Then
                    tw = TextRenderer.MeasureText(line, Me.Font).Width
                End If
            Next
            Dim x As Integer
            Dim y As Integer
            If pointerAngle() > 180 Then
                x = ah + CInt(((img.Width - (ah - 5)) - tw) / 2)
                y = aw + CInt((v - th) / 2)
            Else
                x = CInt(((img.Width - (ah - 5)) - tw) / 2) - img.Width
                y = aw + CInt((v - th) / 2) - img.Height
            End If

            Dim m As New Drawing2D.Matrix
            If pointerAngle() < 180 Then
                m.Rotate(180)
                gr.Transform = m
            End If

            For Each line As String In s
                gr.DrawString(line, Me.Font, New SolidBrush(DirectCast(cboTextColor.SelectedItem, ColorComboBoxItem).color), x, y)
                y += h
            Next

        End If

        Return img

    End Function


    ''' <summary>
    ''' outputImage() Function
    ''' </summary>
    ''' <returns>A trimmed bitmap containing the arrow pointer, as designed
    ''' (for saving as a PNG)</returns>
    Private Function outputImage() As Bitmap
        Dim img As New Bitmap(751, 751)
        Dim gr As Graphics = Graphics.FromImage(img)
        gr.CompositingQuality = Drawing2D.CompositingQuality.HighQuality
        gr.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic

        gr.TranslateTransform(376, 376)
        gr.RotateTransform(Me.angle - 90)
        Dim img2 As Bitmap = arrowImage()
        gr.DrawImage(img2, New Point(0, -(Math.Floor(img2.Height / 2) + 1)))
        gr.ResetTransform()

        Dim width As Integer = img.Width
        Dim height As Integer = img.Height
        Dim left As Integer = width
        Dim right As Integer = 0
        Dim top As Integer = height
        Dim bottom As Integer = 0

        For x As Integer = 0 To width - 1
            For y As Integer = 0 To height - 1
                If img.GetPixel(x, y).A <> 0 Then
                    If y < top Then top = y
                    If y > bottom Then bottom = y
                    If x < left Then left = x
                    If x > right Then right = x
                End If
            Next
        Next

        Dim img3 = New Bitmap(right - left, bottom - top)
        gr = Graphics.FromImage(img3)
        gr.DrawImage(img, New Rectangle(Point.Empty, img3.Size), New Rectangle(left, top, right - left, bottom - top), GraphicsUnit.Pixel)

        Return img3

    End Function
End Class
